==> binnie-mods-1.12-2.2.0.6-sources.jar: binnie/botany/modules/ModuleFlowers.java <==
package binnie.botany.modules;

import binnie.core.util.TileUtil;
import com.google.common.collect.ImmutableSet;

import javax.annotation.Nullable;
import java.awt.Color;
import java.util.Set;
import java.util.function.Predicate;

import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.player.BonemealEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.event.world.BlockEvent;

import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import forestry.api.apiculture.FlowerManager;
import forestry.api.core.ForestryAPI;
import forestry.api.genetics.AlleleManager;
import forestry.api.storage.BackpackManager;
import forestry.api.storage.IBackpackInterface;
import forestry.storage.BackpackDefinition;

import binnie.Constants;
import binnie.botany.Botany;
import binnie.botany.CreativeTabBotany;
import binnie.botany.api.BotanyAPI;
import binnie.botany.api.gardening.IBlockSoil;
import binnie.botany.api.genetics.EnumFlowerStage;
import binnie.botany.api.genetics.IFlower;
import binnie.botany.api.genetics.IFlowerRoot;
import binnie.botany.blocks.BlockFlower;
import binnie.botany.core.BotanyCore;
import binnie.botany.genetics.EnumFlowerColor;
import binnie.botany.genetics.FlowerDefinition;
import binnie.botany.genetics.FlowerFactory;
import binnie.botany.genetics.allele.AlleleEffectNone;
import binnie.botany.items.ItemFlowerGE;
import binnie.botany.tile.TileEntityFlower;
import binnie.core.BinnieCore;
import binnie.core.util.RecipeUtil;
import binnie.modules.BinnieModule;
import binnie.modules.Module;

@BinnieModule(moduleID = BotanyModuleUIDs.FLOWERS, moduleContainerID = Constants.BOTANY_MOD_ID, name = "Flowers", unlocalizedDescription = "botany.module.flowers")
public class ModuleFlowers extends Module {
	public static AlleleEffectNone alleleEffectNone = new AlleleEffectNone();
	@Nullable
	public static Item botanistBackpack;
	/* MODULE GENETIC */
	public static BlockFlower flower;
	public static ItemFlowerGE flowerItem;
	public static ItemFlowerGE seed;
	public static ItemFlowerGE pollen;

	@Override
	public void setupAPI() {
		BotanyAPI.flowerFactory = new FlowerFactory();
		AlleleManager.alleleRegistry.registerSpeciesRoot(BotanyCore.getFlowerRoot());
	}

	@Override
	public void registerItemsAndBlocks() {
		AlleleManager.alleleRegistry.registerAllele(ModuleFlowers.alleleEffectNone);
		EnumFlowerColor.setupMutations();
		FlowerDefinition.preInitFlowers();

		IBackpackInterface backpackInterface = BackpackManager.backpackInterface;
		if (ForestryAPI.enabledPlugins.contains("forestry.storage")) {
			Predicate<ItemStack> filter = BackpackManager.backpackInterface.createNaturalistBackpackFilter("rootFlowers");
			BackpackDefinition definition = new BackpackDefinition(new Color(0xf6e83e), Color.WHITE, filter);
			BackpackManager.backpackInterface.registerBackpackDefinition("botanist", definition);
			botanistBackpack = backpackInterface.createNaturalistBackpack("botanist", BotanyAPI.flowerRoot).setRegistryName("botanist_bag").func_77655_b("botany.botanist_bag");
			Botany.proxy.registerItem(botanistBackpack);
			botanistBackpack.func_77637_a(CreativeTabBotany.instance);
		} else {
			botanistBackpack = null;
		}

		/* ITEMS */
		flower = new BlockFlower();
		flowerItem = new ItemFlowerGE("itemFlower", EnumFlowerStage.FLOWER, "");
		pollen = new ItemFlowerGE("pollen", EnumFlowerStage.POLLEN, "pollen");
		seed = new ItemFlowerGE("seed", EnumFlowerStage.SEED, "germling");

		Botany.proxy.registerBlock(flower);
		Botany.proxy.registerItem(flowerItem);
		Botany.proxy.registerItem(pollen);
		Botany.proxy.registerItem(seed);

		BinnieCore.getBinnieProxy().registerTileEntity(TileEntityFlower.class, "botany.tile.flower", null);
	}

	@Override
	public void preInit() {
		MinecraftForge.EVENT_BUS.register(this);
	}

	@Override
	public void init() {
		EnumFlowerColor.initColours();
		FlowerDefinition.initFlowers();
	}

	@Override
	public void postInit() {
		RecipeUtil recipeUtil = new RecipeUtil(Constants.GENETICS_MOD_ID);
		FlowerManager.flowerRegistry.registerAcceptableFlower(flower, "flowersVanilla");
		recipeUtil.addRecipe("botanist_backpack", botanistBackpack,
			"X#X",
			"VYZ",
			"X#X",
			'#', Blocks.field_150325_L,
			'X', Items.field_151007_F,
			'V', ModuleGardening.soilMeter,
			'Z', "toolTrowel",
			'Y', "chestWood"
		);
	}

	@SubscribeEvent
	public void onShearFlower(PlayerInteractEvent.RightClickBlock event) {
		EntityPlayer player = event.getEntityPlayer();
		if (player == null) {
			return;
		}

		ItemStack heldItem = player.func_184614_ca();
		if (heldItem.func_190926_b()) {
			return;
		}

		TileEntity tile = event.getWorld().func_175625_s(event.getPos());
		if (!(tile instanceof TileEntityFlower)) {
			return;
		}

		TileEntityFlower flower = (TileEntityFlower) tile;
		if (heldItem.func_77973_b() == Items.field_151097_aZ) {
			flower.onShear();
			heldItem.func_77972_a(1, player);
		} else if (heldItem.func_77973_b() == pollen) {
			IFlower pollen = BotanyCore.getFlowerRoot().getMember(heldItem);
			if (pollen != null && flower.canMateWith(pollen)) {
				flower.mateWith(pollen);
				if (!player.field_71075_bZ.field_75098_d) {
					heldItem.func_190918_g(1);
				}
			}
		}
	}

	@SubscribeEvent
	public void plantVanilla(BlockEvent.PlaceEvent event) {
		World world = event.getWorld();
		BlockPos pos = event.getPos();
		Block block = world.func_180495_p(pos.func_177977_b()).func_177230_c();
		if (!BotanyCore.getGardening().isSoil(block)) {
			return;
		}

		EntityPlayer player = event.getPlayer();
		ItemStack heldItem = player.func_184586_b(event.getHand());
		IFlowerRoot flowerRoot = BotanyCore.getFlowerRoot();
		IFlower flower = flowerRoot.getConversion(heldItem);
		if (flower != null) {
			flowerRoot.plant(world, pos, flower, player.func_146103_bH());
		}
	}

	@Deprecated
	public void onPlantVanilla(PlayerInteractEvent.RightClickBlock event) {
		BlockPos pos = event.getPos();
		World world = event.getWorld();
		EntityPlayer player = event.getEntityPlayer();
		ItemStack heldItem = player.func_184614_ca();
		if (!BinnieCore.getBinnieProxy().isSimulating(event.getWorld())) {
			return;
		}

		if (heldItem.func_190926_b()) {
			return;
		}

		Block block = world.func_180495_p(pos).func_177230_c();
		int py = -1;
		if (block instanceof IBlockSoil && (world.func_175623_d(pos.func_177984_a()) || block.func_176200_f(world, pos))) {
			py = 1;
		}
		if (py < 0) {
			return;
		}

		IFlowerRoot flowerRoot = BotanyCore.getFlowerRoot();
		IFlower flower = flowerRoot.getConversion(heldItem);
		if (flower != null && flowerRoot.plant(world, pos.func_177982_a(0, py, 0), flower, player.func_146103_bH()) && !player.field_71075_bZ.field_75098_d) {
			heldItem.func_190918_g(1);
		}
	}

	@SubscribeEvent
	public void onBonemeal(BonemealEvent event) {
		BlockPos pos = event.getPos();

		TileEntityFlower tile = TileUtil.getTile(event.getWorld(), pos, TileEntityFlower.class);
		if (tile != null && tile.onBonemeal()) {
			event.setResult(Event.Result.ALLOW);
		}
	}

	@Override
	public Set<String> getDependencyUids() {
		return ImmutableSet.of(BotanyModuleUIDs.GARDENING);
	}
}

==> binnie-mods-1.12.2-2.4.0.80-sources.jar: binnie/botany/modules/ModuleFlowers.java <==
package binnie.botany.modules;

import com.google.common.collect.ImmutableSet;

import javax.annotation.Nullable;
import java.awt.Color;
import java.util.Set;
import java.util.function.Predicate;

import net.minecraft.block.Block;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.event.entity.player.BonemealEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.event.world.BlockEvent;

import net.minecraftforge.fml.common.eventhandler.Event;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import forestry.api.apiculture.FlowerManager;
import forestry.api.core.ForestryAPI;
import forestry.api.genetics.AlleleManager;
import forestry.api.storage.BackpackManager;
import forestry.api.storage.IBackpackInterface;
import forestry.storage.BackpackDefinition;

import binnie.botany.Botany;
import binnie.botany.CreativeTabBotany;
import binnie.botany.api.BotanyAPI;
import binnie.botany.api.gardening.IBlockSoil;
import binnie.botany.api.genetics.EnumFlowerStage;
import binnie.botany.api.genetics.IFlower;
import binnie.botany.api.genetics.IFlowerRoot;
import binnie.botany.blocks.BlockFlower;
import binnie.botany.core.BotanyCore;
import binnie.botany.genetics.FlowerBreedingSystem;
import binnie.botany.genetics.FlowerColorMutations;
import binnie.botany.genetics.FlowerDefinition;
import binnie.botany.genetics.FlowerFactory;
import binnie.botany.genetics.allele.AlleleEffectNone;
import binnie.botany.items.ItemFlowerGE;
import binnie.botany.tile.TileEntityFlower;
import binnie.core.Binnie;
import binnie.core.BinnieCore;
import binnie.core.Constants;
import binnie.core.api.genetics.IBreedingSystem;
import binnie.core.modules.BinnieModule;
import binnie.core.modules.BotanyModuleUIDs;
import binnie.core.modules.Module;
import binnie.core.util.RecipeUtil;
import binnie.core.util.TileUtil;

@BinnieModule(moduleID = BotanyModuleUIDs.FLOWERS, moduleContainerID = Constants.BOTANY_MOD_ID, name = "Flowers", unlocalizedDescription = "botany.module.flowers")
public class ModuleFlowers implements Module {
	public static final AlleleEffectNone alleleEffectNone = new AlleleEffectNone();
	@Nullable
	public static Item botanistBackpack;
	/* MODULE GENETIC */
	public static BlockFlower flower;
	public static ItemFlowerGE flowerItem;
	public static ItemFlowerGE seed;
	public static ItemFlowerGE pollen;
	public static IBreedingSystem flowerBreedingSystem;

	@Override
	public void setupAPI() {
		BotanyAPI.flowerFactory = new FlowerFactory();
		AlleleManager.alleleRegistry.registerSpeciesRoot(BotanyCore.getFlowerRoot());
	}

	@Override
	public void registerItemsAndBlocks() {
		AlleleManager.alleleRegistry.registerAllele(ModuleFlowers.alleleEffectNone);
		FlowerColorMutations.registerFlowerColorMutations();
		FlowerDefinition.preInitFlowers();

		IBackpackInterface backpackInterface = BackpackManager.backpackInterface;
		if (ForestryAPI.enabledPlugins.contains("forestry.storage")) {
			Predicate<ItemStack> filter = BackpackManager.backpackInterface.createNaturalistBackpackFilter("rootFlowers");
			BackpackDefinition definition = new BackpackDefinition(new Color(0xf6e83e), Color.WHITE, filter);
			BackpackManager.backpackInterface.registerBackpackDefinition("botanist", definition);
			botanistBackpack = backpackInterface.createNaturalistBackpack("botanist", BotanyAPI.flowerRoot).setRegistryName("botanist_bag").setUnlocalizedName("botany.botanist_bag");
			Botany.proxy.registerItem(botanistBackpack);
			botanistBackpack.setCreativeTab(CreativeTabBotany.INSTANCE);
		} else {
			botanistBackpack = null;
		}

		/* ITEMS */
		flower = new BlockFlower();
		flowerItem = new ItemFlowerGE("itemFlower", EnumFlowerStage.FLOWER, "");
		pollen = new ItemFlowerGE("pollen", EnumFlowerStage.POLLEN, "pollen");
		seed = new ItemFlowerGE("seed", EnumFlowerStage.SEED, "germling");

		Botany.proxy.registerBlock(flower);
		Botany.proxy.registerItem(flowerItem);
		Botany.proxy.registerItem(pollen);
		Botany.proxy.registerItem(seed);

		BinnieCore.getBinnieProxy().registerTileEntity(TileEntityFlower.class, "botany.tile.flower", null);
	}

	@Override
	public void preInit() {
		MinecraftForge.EVENT_BUS.register(this);
		flowerBreedingSystem = new FlowerBreedingSystem();
		Binnie.GENETICS.registerBreedingSystem(flowerBreedingSystem);
	}

	@Override
	public void init() {
		FlowerColorMutations.registerFlowerColorAlleles();
		FlowerDefinition.initFlowers();

		RecipeUtil recipeUtil = new RecipeUtil(Constants.BOTANY_MOD_ID);
		FlowerManager.flowerRegistry.registerAcceptableFlower(flower, "flowersVanilla");
		recipeUtil.addRecipe("botanist_backpack", botanistBackpack,
			"X#X",
			"VYZ",
			"X#X",
			'#', Blocks.WOOL,
			'X', Items.STRING,
			'V', ModuleGardening.soilMeter,
			'Z', "toolTrowel",
			'Y', "chestWood"
		);
	}

	@SubscribeEvent
	public void onShearFlower(PlayerInteractEvent.RightClickBlock event) {
		EntityPlayer player = event.getEntityPlayer();
		if (player == null) {
			return;
		}

		ItemStack heldItem = player.getHeldItemMainhand();
		if (heldItem.isEmpty()) {
			return;
		}

		TileEntity tile = event.getWorld().getTileEntity(event.getPos());
		if (!(tile instanceof TileEntityFlower)) {
			return;
		}

		TileEntityFlower flower = (TileEntityFlower) tile;
		if (heldItem.getItem() == Items.SHEARS) {
			flower.onShear();
			heldItem.damageItem(1, player);
		} else if (heldItem.getItem() == pollen) {
			IFlower pollen = BotanyCore.getFlowerRoot().getMember(heldItem);
			if (pollen != null && flower.canMateWith(pollen)) {
				flower.mateWith(pollen);
				if (!player.capabilities.isCreativeMode) {
					heldItem.shrink(1);
				}
			}
		}
	}

	@SubscribeEvent
	public void plantVanilla(BlockEvent.PlaceEvent event) {
		World world = event.getWorld();
		BlockPos pos = event.getPos();
		Block block = world.getBlockState(pos.down()).getBlock();
		if (!BotanyCore.getGardening().isSoil(block)) {
			return;
		}

		EntityPlayer player = event.getPlayer();
		ItemStack heldItem = player.getHeldItem(event.getHand());
		IFlowerRoot flowerRoot = BotanyCore.getFlowerRoot();
		IFlower flower = flowerRoot.getConversion(heldItem);
		if (flower != null) {
			flowerRoot.plant(world, pos, flower, player.getGameProfile());
		}
	}

	@Deprecated
	public void onPlantVanilla(PlayerInteractEvent.RightClickBlock event) {
		BlockPos pos = event.getPos();
		World world = event.getWorld();
		EntityPlayer player = event.getEntityPlayer();
		ItemStack heldItem = player.getHeldItemMainhand();
		if (!!event.getWorld().isRemote) {
			return;
		}

		if (heldItem.isEmpty()) {
			return;
		}

		Block block = world.getBlockState(pos).getBlock();
		int py = -1;
		if (block instanceof IBlockSoil && (world.isAirBlock(pos.up()) || block.isReplaceable(world, pos))) {
			py = 1;
		}
		if (py < 0) {
			return;
		}

		IFlowerRoot flowerRoot = BotanyCore.getFlowerRoot();
		IFlower flower = flowerRoot.getConversion(heldItem);
		if (flower != null && flowerRoot.plant(world, pos.add(0, py, 0), flower, player.getGameProfile()) && !player.capabilities.isCreativeMode) {
			heldItem.shrink(1);
		}
	}

	@SubscribeEvent
	public void onBonemeal(BonemealEvent event) {
		BlockPos pos = event.getPos();

		TileEntityFlower tile = TileUtil.getTile(event.getWorld(), pos, TileEntityFlower.class);
		if (tile != null && tile.onBonemeal()) {
			event.setResult(Event.Result.ALLOW);
		}
	}

	@Override
	public Set<String> getDependencyUids() {
		return ImmutableSet.of(BotanyModuleUIDs.GARDENING);
	}
}
